"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.PopoverView = void 0;

var _react = _interopRequireDefault(require("react"));

var _reactNative = require("react-native");

var _RNIPopoverView = require("../../native_components/RNIPopoverView");

var _RNIPopoverViewModule = require("../../native_modules/RNIPopoverViewModule");

var Helpers = _interopRequireWildcard(require("../../functions/helpers"));

var _LibEnv = require("../../constants/LibEnv");

var _reactNativeIosUtilities = require("react-native-ios-utilities");

var _RNIPopoverErrorCodes = require("../../constants/RNIPopoverErrorCodes");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const NATIVE_ID_KEYS = {
  popoverWrapperView: 'popoverContentView'
};

class PopoverView extends _react.default.PureComponent {
  constructor(props) {
    var _this;

    super(props);
    _this = this;

    _defineProperty(this, "nativeRef", void 0);

    _defineProperty(this, "getProps", () => {
      const {
        popoverBackgroundColor,
        permittedArrowDirections,
        popoverSize,
        popoverShouldDismiss,
        popoverCanOverlapSourceViewRect,
        lazyPopover,
        internalCleanupMode,
        renderPopoverContent,
        onPopoverDidHide,
        onPopoverWillShow,
        onPopoverWillHide,
        onPopoverDidShow,
        onPopoverDidHideViaTap,
        onPopoverWillHideViaTap,
        onPopoverDidAttemptToDismiss,
        ...viewProps
      } = this.props;
      return {
        // A. Provide default values to props...
        lazyPopover: lazyPopover !== null && lazyPopover !== void 0 ? lazyPopover : true,
        popoverShouldDismiss: popoverShouldDismiss !== null && popoverShouldDismiss !== void 0 ? popoverShouldDismiss : true,
        // B. Pass down props...
        popoverBackgroundColor,
        permittedArrowDirections,
        popoverSize,
        popoverCanOverlapSourceViewRect,
        internalCleanupMode,
        renderPopoverContent,
        onPopoverDidHide,
        onPopoverWillShow,
        onPopoverWillHide,
        onPopoverDidShow,
        onPopoverDidHideViaTap,
        onPopoverWillHideViaTap,
        onPopoverDidAttemptToDismiss,
        // C. Move all the default view-related
        //    props here...
        viewProps
      };
    });

    _defineProperty(this, "setVisibility", async function (visibility) {
      let options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {
        suppressVisibilityError: true
      };

      const {
        lazyPopover
      } = _this.getProps();

      if (!_LibEnv.IS_PLATFORM_IOS) return;

      try {
        if (visibility) {
          await Promise.all([Helpers.setStateAsync(_this, {
            mountPopover: true
          }), // temp bugfix: wait for popover to mount
          lazyPopover && Helpers.timeout(50)]);
        }

        ;
        await _RNIPopoverViewModule.RNIPopoverViewModule.setVisibility((0, _reactNative.findNodeHandle)(_this.nativeRef), visibility);
      } catch (error) {
        if (_reactNativeIosUtilities.ErrorUtilities.isNativeError(error)) {
          const nativeError = error;
          const isVisibilityError = nativeError.code === _RNIPopoverErrorCodes.RNIPopoverErrorCodes.popoverAlreadyHidden || nativeError.code === _RNIPopoverErrorCodes.RNIPopoverErrorCodes.popoverAlreadyVisible;

          if (isVisibilityError && options.suppressVisibilityError) {// no-op - don't show "popover already visible/hidden"-related errors
          } else {
            console.warn(`Code: ${error.code} - Message: ${error.message}`);
            throw error;
          }

          ;
        } else if (error instanceof Error) {
          throw error;
        } else {
          throw error;
        }

        ;
      }

      ;
    });

    _defineProperty(this, "toggleVisibility", async () => {
      if (!_LibEnv.IS_PLATFORM_IOS) return;
      const visibility = await this.getVisibility();
      await this.setVisibility(!visibility);
    });

    _defineProperty(this, "getVisibility", async () => {
      if (!_LibEnv.IS_PLATFORM_IOS) return;
      return await _RNIPopoverViewModule.RNIPopoverViewModule.getVisibility((0, _reactNative.findNodeHandle)(this.nativeRef));
    });

    _defineProperty(this, "_handleOnPopoverDidHide", event => {
      var _this$props$onPopover, _this$props;

      (_this$props$onPopover = (_this$props = this.props).onPopoverDidHide) === null || _this$props$onPopover === void 0 ? void 0 : _this$props$onPopover.call(_this$props, event);
      event.stopPropagation();
      Helpers.setStateAsync(this, {
        mountPopover: false
      });
    });

    this.state = {
      /** controls whether the popover content is mounted */
      mountPopover: false
    };
  }

  componentWillUnmount() {
    _RNIPopoverViewModule.RNIPopoverViewModule.notifyComponentWillUnmount((0, _reactNative.findNodeHandle)(this.nativeRef), {});
  }

  //#endregion
  render() {
    var _props$renderPopoverC;

    const props = this.getProps();
    const {
      mountPopover
    } = this.state;
    return _LibEnv.IS_PLATFORM_IOS ? /*#__PURE__*/_react.default.createElement(_RNIPopoverView.RNIPopoverView, _extends({
      ref: r => {
        this.nativeRef = r;
      }
    }, props.viewProps, {
      // props
      popoverBackgroundColor: (0, _reactNative.processColor)(props.popoverBackgroundColor),
      permittedArrowDirections: props.permittedArrowDirections,
      popoverSize: props.popoverSize,
      popoverShouldDismiss: props.popoverShouldDismiss,
      popoverCanOverlapSourceViewRect: props.popoverCanOverlapSourceViewRect,
      internalCleanupMode: props.internalCleanupMode // events
      ,
      onPopoverWillShow: props.onPopoverWillShow,
      onPopoverWillHide: props.onPopoverWillHide,
      onPopoverDidShow: props.onPopoverDidShow,
      onPopoverDidHide: this._handleOnPopoverDidHide,
      onPopoverDidHideViaTap: props.onPopoverDidHideViaTap,
      onPopoverWillHideViaTap: props.onPopoverWillHideViaTap,
      onPopoverDidAttemptToDismiss: props.onPopoverDidAttemptToDismiss
    }), (mountPopover || !props.lazyPopover) && /*#__PURE__*/_react.default.createElement(_reactNativeIosUtilities.RNIWrapperView, {
      style: styles.popoverContentWrapper,
      nativeID: NATIVE_ID_KEYS.popoverWrapperView,
      isDummyView: true,
      shouldAutoDetachSubviews: true,
      shouldCreateTouchHandlerForSubviews: true,
      shouldNotifyComponentWillUnmount: true,
      shouldAutoCleanupOnJSUnmount: true,
      shouldAutoSetSizeOnLayout: true
    }, (_props$renderPopoverC = props.renderPopoverContent) === null || _props$renderPopoverC === void 0 ? void 0 : _props$renderPopoverC.call(props)), this.props.children) : /*#__PURE__*/_react.default.createElement(_reactNative.View, props.viewProps, this.props.children);
  }

}

exports.PopoverView = PopoverView;
;

const styles = _reactNative.StyleSheet.create({
  popoverContentWrapper: {
    position: 'absolute',
    color: 'transparent'
  }
});
//# sourceMappingURL=PopoverView.js.map