function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import React from 'react';
import { StyleSheet, findNodeHandle, View, processColor } from 'react-native';
import { RNIPopoverView } from '../../native_components/RNIPopoverView';
import { RNIPopoverViewModule } from '../../native_modules/RNIPopoverViewModule';
import * as Helpers from '../../functions/helpers';
import { IS_PLATFORM_IOS } from '../../constants/LibEnv';
import { ErrorUtilities, RNIWrapperView } from 'react-native-ios-utilities';
import { RNIPopoverErrorCodes } from '../../constants/RNIPopoverErrorCodes';
const NATIVE_ID_KEYS = {
  popoverWrapperView: 'popoverContentView'
};
export class PopoverView extends React.PureComponent {
  constructor(props) {
    var _this;

    super(props);
    _this = this;

    _defineProperty(this, "nativeRef", void 0);

    _defineProperty(this, "getProps", () => {
      const {
        popoverBackgroundColor,
        permittedArrowDirections,
        popoverSize,
        popoverShouldDismiss,
        popoverCanOverlapSourceViewRect,
        lazyPopover,
        internalCleanupMode,
        renderPopoverContent,
        onPopoverDidHide,
        onPopoverWillShow,
        onPopoverWillHide,
        onPopoverDidShow,
        onPopoverDidHideViaTap,
        onPopoverWillHideViaTap,
        onPopoverDidAttemptToDismiss,
        ...viewProps
      } = this.props;
      return {
        // A. Provide default values to props...
        lazyPopover: lazyPopover !== null && lazyPopover !== void 0 ? lazyPopover : true,
        popoverShouldDismiss: popoverShouldDismiss !== null && popoverShouldDismiss !== void 0 ? popoverShouldDismiss : true,
        // B. Pass down props...
        popoverBackgroundColor,
        permittedArrowDirections,
        popoverSize,
        popoverCanOverlapSourceViewRect,
        internalCleanupMode,
        renderPopoverContent,
        onPopoverDidHide,
        onPopoverWillShow,
        onPopoverWillHide,
        onPopoverDidShow,
        onPopoverDidHideViaTap,
        onPopoverWillHideViaTap,
        onPopoverDidAttemptToDismiss,
        // C. Move all the default view-related
        //    props here...
        viewProps
      };
    });

    _defineProperty(this, "setVisibility", async function (visibility) {
      let options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {
        suppressVisibilityError: true
      };

      const {
        lazyPopover
      } = _this.getProps();

      if (!IS_PLATFORM_IOS) return;

      try {
        if (visibility) {
          await Promise.all([Helpers.setStateAsync(_this, {
            mountPopover: true
          }), // temp bugfix: wait for popover to mount
          lazyPopover && Helpers.timeout(50)]);
        }

        ;
        await RNIPopoverViewModule.setVisibility(findNodeHandle(_this.nativeRef), visibility);
      } catch (error) {
        if (ErrorUtilities.isNativeError(error)) {
          const nativeError = error;
          const isVisibilityError = nativeError.code === RNIPopoverErrorCodes.popoverAlreadyHidden || nativeError.code === RNIPopoverErrorCodes.popoverAlreadyVisible;

          if (isVisibilityError && options.suppressVisibilityError) {// no-op - don't show "popover already visible/hidden"-related errors
          } else {
            console.warn(`Code: ${error.code} - Message: ${error.message}`);
            throw error;
          }

          ;
        } else if (error instanceof Error) {
          throw error;
        } else {
          throw error;
        }

        ;
      }

      ;
    });

    _defineProperty(this, "toggleVisibility", async () => {
      if (!IS_PLATFORM_IOS) return;
      const visibility = await this.getVisibility();
      await this.setVisibility(!visibility);
    });

    _defineProperty(this, "getVisibility", async () => {
      if (!IS_PLATFORM_IOS) return;
      return await RNIPopoverViewModule.getVisibility(findNodeHandle(this.nativeRef));
    });

    _defineProperty(this, "_handleOnPopoverDidHide", event => {
      var _this$props$onPopover, _this$props;

      (_this$props$onPopover = (_this$props = this.props).onPopoverDidHide) === null || _this$props$onPopover === void 0 ? void 0 : _this$props$onPopover.call(_this$props, event);
      event.stopPropagation();
      Helpers.setStateAsync(this, {
        mountPopover: false
      });
    });

    this.state = {
      /** controls whether the popover content is mounted */
      mountPopover: false
    };
  }

  componentWillUnmount() {
    RNIPopoverViewModule.notifyComponentWillUnmount(findNodeHandle(this.nativeRef), {});
  }

  //#endregion
  render() {
    var _props$renderPopoverC;

    const props = this.getProps();
    const {
      mountPopover
    } = this.state;
    return IS_PLATFORM_IOS ? /*#__PURE__*/React.createElement(RNIPopoverView, _extends({
      ref: r => {
        this.nativeRef = r;
      }
    }, props.viewProps, {
      // props
      popoverBackgroundColor: processColor(props.popoverBackgroundColor),
      permittedArrowDirections: props.permittedArrowDirections,
      popoverSize: props.popoverSize,
      popoverShouldDismiss: props.popoverShouldDismiss,
      popoverCanOverlapSourceViewRect: props.popoverCanOverlapSourceViewRect,
      internalCleanupMode: props.internalCleanupMode // events
      ,
      onPopoverWillShow: props.onPopoverWillShow,
      onPopoverWillHide: props.onPopoverWillHide,
      onPopoverDidShow: props.onPopoverDidShow,
      onPopoverDidHide: this._handleOnPopoverDidHide,
      onPopoverDidHideViaTap: props.onPopoverDidHideViaTap,
      onPopoverWillHideViaTap: props.onPopoverWillHideViaTap,
      onPopoverDidAttemptToDismiss: props.onPopoverDidAttemptToDismiss
    }), (mountPopover || !props.lazyPopover) && /*#__PURE__*/React.createElement(RNIWrapperView, {
      style: styles.popoverContentWrapper,
      nativeID: NATIVE_ID_KEYS.popoverWrapperView,
      isDummyView: true,
      shouldAutoDetachSubviews: true,
      shouldCreateTouchHandlerForSubviews: true,
      shouldNotifyComponentWillUnmount: true,
      shouldAutoCleanupOnJSUnmount: true,
      shouldAutoSetSizeOnLayout: true
    }, (_props$renderPopoverC = props.renderPopoverContent) === null || _props$renderPopoverC === void 0 ? void 0 : _props$renderPopoverC.call(props)), this.props.children) : /*#__PURE__*/React.createElement(View, props.viewProps, this.props.children);
  }

}
;
const styles = StyleSheet.create({
  popoverContentWrapper: {
    position: 'absolute',
    color: 'transparent'
  }
});
//# sourceMappingURL=PopoverView.js.map